unit FindUnit;

interface
uses Classes,Windows, SysUtils, RTLConsts;
type
	TFileFind = Class
	private
		SR : TSearchRec;
		FFinded :Boolean;
		FilePath : String;
		function GetTime: TDateTime;
		function GetFileName: TFileName;
	public
		Constructor Create(const FileName : String;const Attribute : Integer);
		Destructor Destroy;override;
		function FindNext : Boolean;
	published
		property FileName : TFileName read GetFileName;
		property Name : TFileName read SR.Name;
		property Time : TDateTime read GetTime;
		property Attribute : Integer read SR.Attr;
		property Size : Int64 read SR.Size write SR.Size;
		property ExcludeAttr : Integer read SR.ExcludeAttr;  
{$IFDEF MSWINDOWS}
		property Handle : Cardinal read SR.FindHandle;
		property FindData : TWin32FindData read SR.FindData; 
{$ENDIF}
{$IFDEF LINUX}        
		property Handle : Pointer read SR.FindHandle;
		property Mode : mode_t read SR.Mode;
{$ENDIF}
		property Finded : Boolean read FFinded;
	End;

	TWideSearchRec = record
		Time: Integer;
		Size: Int64;
		Attr: Integer;
		Name: WideString;
		ExcludeAttr: Integer;
{$IFDEF MSWINDOWS}
		FindHandle: THandle  platform;
		FindData: TWin32FindDataW  platform;
{$ENDIF}
{$IFDEF LINUX}
		Mode: mode_t  platform;
		FindHandle: Pointer  platform;
		PathOnly: String  platform;
		Pattern: String  platform;
{$ENDIF}
	end;

	TUniFileFind = Class
	private
		SR : TWideSearchRec;
		FFinded :Boolean;
		FilePath : String;
		function GetTime: TDateTime;
		function GetFileName: WideString;
	public
		Constructor Create(const AFilePath : String;const Attribute : Integer);
		Destructor Destroy;override;
		function FindNext : Boolean;
	published
		property FileName : WideString read GetFileName;
		property Name : WideString read SR.Name;
		property Time : TDateTime read GetTime;
		property Attribute : Integer read SR.Attr;
		property Size : Int64 read SR.Size write SR.Size;
		property ExcludeAttr : Integer read SR.ExcludeAttr;
{$IFDEF MSWINDOWS}
		property Handle : Cardinal read SR.FindHandle;
		property FindData : TWin32FindDataW read SR.FindData;
{$ENDIF}
{$IFDEF LINUX}
		property Handle : Pointer read SR.FindHandle;
		property Mode : mode_t read SR.Mode;
{$ENDIF}
		property Finded : Boolean read FFinded;
	End;

	TFileStreamW = class(THandleStream)
	public
		function MeStream: TStream;
		constructor Create(const FileName: WideString; Mode: Word); overload;
		destructor Destroy; override;
	end;

	function LoadFromFileToStr(fname:WideString):string;

implementation

function LoadFromFileToStr(fname:WideString):string;
	var hin:THandle;
	var rsize:DWORD;
begin
	Result:='';
	hin:=CreateFileW(PWideChar(fname),GENERIC_READ,0,nil,OPEN_EXISTING ,FILE_ATTRIBUTE_NORMAL,0);
	if hin<>0 then try
		SetLength(Result, GetFileSize(hin, nil));
		ReadFile(hin,PChar(Result)^,Length(Result),rsize , nil);
		finally CloseHandle(hin);
		end
end;

{ TFileFind }

constructor TFileFind.Create(const FileName: String; const Attribute: Integer);
begin
	FilePath := ExtractFilePath(FileName);
	if FilePath = FileName then
		FFinded := FindFirst(IncludeTrailingPathDelimiter(FileName) + '*.*',Attribute,SR) = 0
	else FFinded := FindFirst(FileName,Attribute,SR) = 0;
end;

destructor TFileFind.Destroy;
begin
	FindClose(SR);
	inherited;
end;

function TFileFind.FindNext: Boolean;
begin
	Result := SysUtils.FindNext(SR) = 0;
	FFinded := Result;
end;

function TFileFind.GetFileName: TFileName;
begin
	Result := FilePath + SR.Name;
end;

function TFileFind.GetTime: TDateTime;
begin
	Result := FileDateToDateTime(SR.Time);
end;

{ TUniFileFind }

procedure FindCloseW(var F: TWideSearchRec);
begin
{$IFDEF MSWINDOWS}
  if F.FindHandle <> INVALID_HANDLE_VALUE then
  begin
    Windows.FindClose(F.FindHandle);
    F.FindHandle := INVALID_HANDLE_VALUE;
  end;
{$ENDIF}
{$IFDEF LINUX}
  if F.FindHandle <> nil then
  begin
    closedir(F.FindHandle);
    F.FindHandle := nil;
  end;
{$ENDIF}
end;

function FindMatchingFileW(var F: TWideSearchRec): Integer;
{$IFDEF MSWINDOWS}
var
	LocalFileTime: TFileTime;
begin
	with F do
	begin
		while FindData.dwFileAttributes and ExcludeAttr <> 0 do
			if not FindNextFileW(FindHandle, FindData) then
			begin
				Result := GetLastError;
				Exit;
			end;
		FileTimeToLocalFileTime(FindData.ftLastWriteTime, LocalFileTime);
		FileTimeToDosDateTime(LocalFileTime, LongRec(Time).Hi,
			LongRec(Time).Lo);
		Size := FindData.nFileSizeLow or Int64(FindData.nFileSizeHigh) shl 32;
		Attr := FindData.dwFileAttributes;
		Name := FindData.cFileName;
	end;
	Result := 0;
end;
{$ENDIF}
{$IFDEF LINUX}
var
	PtrDirEnt: PDirEnt;
	Scratch: TDirEnt;
	StatBuf: TStatBuf;
	LinkStatBuf: TStatBuf;
	FName: string;
	Attr: Integer;
	Mode: mode_t;
begin
	Result := -1;
	PtrDirEnt := nil;
	if readdir_r(F.FindHandle, @Scratch, PtrDirEnt) <> 0 then
		Exit;
	while PtrDirEnt <> nil do
	begin
		if fnmatch(PChar(F.Pattern), PtrDirEnt.d_name, 0) = 0 then
		begin   // F.PathOnly must include trailing backslash
			FName := F.PathOnly + string(PtrDirEnt.d_name);

			if lstat(PChar(FName), StatBuf) = 0 then
			begin
				Attr := 0;
				Mode := StatBuf.st_mode;

				if S_ISDIR(Mode) then
					Attr := Attr or faDirectory
				else
				if not S_ISREG(Mode) then  // directories shouldn't be treated as system files
				begin
					if S_ISLNK(Mode) then
					begin
						Attr := Attr or faSymLink;
						if (stat(PChar(FName), LinkStatBuf) = 0) and
							S_ISDIR(LinkStatBuf.st_mode) then
								Attr := Attr or faDirectory
					end;
					Attr := Attr or faSysFile;
				end;

				if (PtrDirEnt.d_name[0] = '.') and (PtrDirEnt.d_name[1] <> #0) then
				begin
					if not ((PtrDirEnt.d_name[1] = '.') and (PtrDirEnt.d_name[2] = #0)) then
						Attr := Attr or faHidden;
				end;

				if euidaccess(PChar(FName), W_OK) <> 0 then
					Attr := Attr or faReadOnly;

				if Attr and F.ExcludeAttr = 0 then
				begin
					F.Size := StatBuf.st_size;
					F.Attr := Attr;
					F.Mode := StatBuf.st_mode;
					F.Name := PtrDirEnt.d_name;
					F.Time := StatBuf.st_mtime;
					Result := 0;
					Break;
				end;
			end;
		end;
		Result := -1;
		if readdir_r(F.FindHandle, @Scratch, PtrDirEnt) <> 0 then
			Break;
	end // End of While
end;
{$ENDIF}  

function FindNextW(var F: TWideSearchRec): Integer;
begin
{$IFDEF MSWINDOWS}
	if FindNextFileW(F.FindHandle, F.FindData) then
		Result := FindMatchingFileW(F) else
		Result := GetLastError;
{$ENDIF}
{$IFDEF LINUX}
	Result := FindMatchingFile(F);
{$ENDIF}
end;

function FindFirstW(const Path: Widestring; Attr: Integer;
	var  F: TWideSearchRec): Integer;
const
	faSpecial = faHidden or faSysFile or faDirectory;
{$IFDEF MSWINDOWS}
begin
	F.ExcludeAttr := not Attr and faSpecial;
	F.FindHandle := FindFirstFileW(PWideChar(Path), F.FindData);
	if F.FindHandle <> INVALID_HANDLE_VALUE then
	begin
		Result := FindMatchingFileW(F);
		if Result <> 0 then FindCloseW(F);
	end else
		Result := GetLastError;
end;
{$ENDIF}
{$IFDEF LINUX}
begin
	F.ExcludeAttr := not Attr and faSpecial;
	F.PathOnly := ExtractFilePath(Path);
	F.Pattern := ExtractFileName(Path);
	if F.PathOnly = '' then
		F.PathOnly := IncludeTrailingPathDelimiter(GetCurrentDir);

	F.FindHandle := opendir(PChar(F.PathOnly));
	if F.FindHandle <> nil then
	begin
		Result := FindMatchingFile(F);
		if Result <> 0 then
			FindClose(F);
	end
	else
		Result:= GetLastError;
end;
{$ENDIF}

constructor TUniFileFind.Create(const AFilePath: String;
	const Attribute: Integer);
begin
	FilePath := ExtractFilePath(AFilePath);
	if FilePath = AFilePath then
		FFinded := FindFirstW(IncludeTrailingPathDelimiter(FileName) + '*.*',Attribute,SR) = 0
	else FFinded := FindFirstW(FileName,Attribute,SR) = 0;
end;

destructor TUniFileFind.Destroy;
begin
	FindCloseW(SR);
	inherited;
end;

function TUniFileFind.FindNext: Boolean;
begin
	Result := FindNextW(SR) = 0;
	FFinded := Result;
end;

function TUniFileFind.GetFileName: WideString;
begin
	Result := FilePath + SR.Name;
end;

function TUniFileFind.GetTime: TDateTime;
begin
	Result := FileDateToDateTime(SR.Time);
end;
	// uses Consts;
	// o[W̏ꍇ uses RTLConsts;

constructor TFileStreamW.Create(const FileName: WideString; Mode: Word);
	function FileCreateW(const FileName: WideString): Integer;
	begin
		Result := Integer(CreateFileW(PWideChar(FileName), GENERIC_READ or
			GENERIC_WRITE,
			0, nil, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0));
	end;

	function FileOpenW(const FileName: WideString; Mode: LongWord): Integer;
	const
		AccessMode: array[0..2] of LongWord = (
			GENERIC_READ,
			GENERIC_WRITE,
			GENERIC_READ or GENERIC_WRITE);
		ShareMode: array[0..4] of LongWord = (
			0,
			0,
			FILE_SHARE_READ,
			FILE_SHARE_WRITE,
      FILE_SHARE_READ or FILE_SHARE_WRITE);
	begin
		Result := Integer(CreateFileW(PWideChar(FileName), AccessMode[Mode and 3],
			ShareMode[(Mode and $F0) shr 4], nil, OPEN_EXISTING,
      FILE_ATTRIBUTE_NORMAL, 0));
	end;

var
  aHandle: Integer;
begin
  if Mode = fmCreate then
	begin
		aHandle := FileCreateW(FileName);
		if aHandle < 0 then
			raise EFCreateError.CreateResFmt(@SFCreateError, [FileName]);
	end
  else
	begin
		aHandle := FileOpenW(FileName, Mode);
		if aHandle < 0 then
			raise EFOpenError.CreateResFmt(@SFOpenError, [FileName]);
	end;
  inherited Create(aHandle);
end;

destructor TFileStreamW.Destroy;
begin
	if Handle >= 0 then
		CloseHandle(THandle(Handle));
end;

function TFileStreamW.MeStream: TStream;
begin
	Result := Self;
end;


end.